/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005, CAHA
! IDENT        Mosaic_cubes.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      Mosaic Cubes
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  2002-Dec-17 : Creation, SF.
------------------------------------------------------------------------------*/

#include <IFU_io.h>

#include "stats.h"

/*-----------------------------------------------------------------------------
!
!.blk            Transform the PMAS reduced data to Euro3D data: No GROUP info included.
!
!.prog                             pmas2Euro3D()
!
!.purp         Facility to transform PMAS reduced data to Euro3D data format.
!
-----------------------------------------------------------------------------*/

#define MAX_MOSAICS 30



main(int argc, char **argv)
{

  FILE *config_id;  
  char config[80];
  char fitsfiles[MAX_MOSAICS][80];
  float dx[MAX_MOSAICS],dy[MAX_MOSAICS];
  float dxmin,dxmax,dymin,dymax;
  int i,j,k,n,n_fits;
  float tmp_f;
  char input_filename[80];
  char output_filename[80];
  double crval[3];
  double cdelt[3];
  IMAGE3D cube;
  float *out_array;
  int *sum;
  long npix;
  int nx_start,ny_start;
  int ii,jj,kk,j_file;
  char **argval, **arglabel;
  long npix_out[3];
  

  if (argc<4) {
      printf("USE: Mosaic_cubes CONFIG outfile nfits [nx ny]\n");
      return(0);
  }
/*
 * Reading the input parameters
 */
  strcpy(config,argv[1]);
  strcpy(output_filename,argv[2]);
  n_fits=atoi(argv[3]);
  if (argc>4) {
      npix_out[0]=atoi(argv[4]);
      npix_out[1]=atoi(argv[5]);
  }
  init_session(argv,argc,&arglabel,&argval);

/*
 * Reading the config file
 */  
  dxmin=1e12;
  dymin=1e12;
  printf("Config File : %s\n",config);
  if ((config_id=fopen(config,"r"))==NULL) {
      printf("Error opening file: %s\n",config);
      return(-1);
  } else {    
      for (j=0;j<n_fits;j++) {
	  fscanf(config_id, "%s", &fitsfiles[j]);
	  fscanf(config_id, "%f", &dx[j]);
	  fscanf(config_id, "%f", &dy[j]);
	  dx[j]=dx[j];
	  dy[j]=dy[j];
	  if (dxmin>dx[j]) {
	      dxmin=dx[j];
	  }
	  if (dxmax<dx[j]) {
	      dxmax=dx[j];
	  }
	  if (dymin>dy[j]) {
	      dymin=dy[j];
	  }
	  if (dymax<dy[j]) {
	      dymax=dy[j];
	  }
      }
  }

/*
 * Reading the cubes 
 */  
  for (j_file=0;j_file<n_fits;j_file++) {
      printf("Adding %s...",fitsfiles[j_file]);
      open_cube(&cube,fitsfiles[j_file],"I");
      fflush(stdout);
      if (j_file==0) {
	  if(!RD_desc(&cube,"CRVAL1",DOUBLE,1,&crval[0])) {
	      crval[0]=1;
	  } 
	  if(!RD_desc(&cube,"CRVAL2",DOUBLE,1,&crval[1])) {
	      crval[1]=1;
	  } 
	  if(!RD_desc(&cube,"CRVAL3",DOUBLE,1,&crval[2])) {
	      crval[2]=1;
	  } 
	  if(!RD_desc(&cube,"CDELT1",DOUBLE,1,&cdelt[0])) {
	      cdelt[0]=1;
	  } 
	  if(!RD_desc(&cube,"CDELT2",DOUBLE,1,&cdelt[1])) {
	      cdelt[1]=1;
	  } 
	  if(!RD_desc(&cube,"CDELT3",DOUBLE,1,&cdelt[2])) {
	      cdelt[2]=1;
	  } 

/*
	  RD_desc(&cube,"CRVAL1",DOUBLE,1,&crval[0]);
	  RD_desc(&cube,"CRVAL2",DOUBLE,1,&crval[1]);
	  RD_desc(&cube,"CRVAL3",DOUBLE,1,&crval[2]);
	  RD_desc(&cube,"CDELT1",DOUBLE,1,&cdelt[1]);
	  RD_desc(&cube,"CDELT2",DOUBLE,1,&cdelt[2]);
	  RD_desc(&cube,"CDELT3",DOUBLE,1,&cdelt[3]);
*/
	  fflush(stdout);
	  if (argc<5) {
	      npix_out[0]=cube.nx; 
	      npix_out[1]=cube.ny; 
	  }	  
	  npix_out[2]=cube.nz;

	  npix=npix_out[0]*npix_out[1]*cube.nz;
	  out_array = (float *) calloc(npix,sizeof(float));    	  
	  sum = (int *) calloc(npix,sizeof(int));    	  

	  n=0;
	  for (k=0;k<cube.nz;k++) {
	      for (j=0;j<npix_out[1];j++) {
		  for (i=0;i<npix_out[0];i++) {
		      sum[n]=0;
		      out_array[n]=0;		      
		      n++;
		  }
	      }
	  }



      }      
      n=0;
      for (k=0;k<cube.nz;k++) {
	  for (jj=0;jj<npix_out[1];jj++) {
	      for (ii=0;ii<npix_out[0];ii++) {
		  /*
		  ii=i-floor(dx[j_file]-dxmin);
		  jj=j-floor(dy[j_file]-dymin);
		  */
		  i=ii-floor(dx[j_file]-dxmin);
		  j=jj-floor(dy[j_file]-dymin);
		  //if ((ii>=0)&&(ii<npix_out[0])&&(jj>=0)&&(jj<npix_out[0])) {
		  if ((i>=0)&&(i<cube.nx)&&(j>=0)&&(j<cube.ny)) {
		      out_array[n]=out_array[n]+(float)RD_cube(&cube,i,j,k);
		      sum[n]=sum[n]+1;
		  } 
//		  printf("%d %d %d %d %f %d\n",i,j,ii,jj,out_array[n],sum[n]);

		  n++;
	      }
	  }
      }    
      
      printf("added at %f,%f\n",dx[j_file],dy[j_file]);
      close_cube(&cube);
      }
      
      for (j=0;j<n;j++) {
	  if (sum[j]>0) {
	      out_array[j]=out_array[j]/sum[j];
	  } else {
	      out_array[j]=0;
	  }
      }

      create_cube(&cube,output_filename,npix_out,crval,cdelt,FLOAT,"Mosaic Cube",NULL);
      n=0;
      for (k=0;k<cube.nz;k++) {
	  for (j=0;j<npix_out[1];j++) {
	      for (i=0;i<npix_out[0];i++) {
		  WR_cube(&cube,i,j,k,out_array[n]);
		  n++;
	      }
	  }
      }      
      close_cube(&cube); 
      
      exit_session(0);
      return(0);	
}



