/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
! COPYRIGHT    (c)  2005 CAHA, Spain
! IDENT        trace_peaks_recursive.c
! LANGUAGE     C
! AUTHOR       S.F.Sanchez
! KEYWORDS     
! PURPOSE      C version of trace_peaks_recursive.pl
! COMMENT      No GROUP information included. No header data properly copied
! VERSION      0.1  08.08.05 : Creation, SF.
------------------------------------------------------------------------------*/

#include <IFU_io.h>

#include "stats.h"
#include <stdlib.h>


#include <stdio.h> //For Standard I/O
#include <ctype.h> //For Type check
#include <string.h> //For String manipulation
#include <math.h> //For Math functions
#include <data_io.h> //For Lyon I/O data

#include <cpgplot.h> // PGPLOT


main(int argc, char *ARGV[])
{
    FILE *FH;
    char infile[80],peaks_file[80];
    int spec_axis;
    int plot,nplot,nsearch,width,nx,ny;
    float dmin,imin;
    long npix[2];
    double start[2];
    double step[2];
    IMAGE2D image;
    float *sec_array,*cut;
    int mm,i,j,k;
    float fmin,fmax;
    int *peak_y_pixel;
    float *peak_y_max;
    int npeaks,peak;
    float delta;
    int a,b,c;
    float fa,fb,fc,den;
    float *x,*y;

    if (argc<9) {
	printf("USE: peak_find RAW.fits Spectral_axis[0/1] Coadd_width plot nplot nsearch DMIN IMIN(% of the MAX) OUTFILE [colum_search]\n");
	return(0);
    }
    set_user_dataformat();
    strcpy(infile,ARGV[1]);
    spec_axis=atoi(ARGV[2]);
    width=atoi(ARGV[3]);
    plot=atoi(ARGV[4]);
    nplot=atoi(ARGV[5]);
    nsearch=atoi(ARGV[6]);
    dmin=atof(ARGV[7]);
    imin=atof(ARGV[8]);
    strcpy(peaks_file,ARGV[9]);


    printf("Reading file infile '%s'",infile);


    open_frame(&image,infile,"I");
    if (spec_axis==0) {
	npix[0]=image.nx;
	npix[1]=image.ny;
	nx=image.nx;
	ny=image.ny;
    } else {
	npix[0]=image.ny;
	npix[1]=image.nx;
	nx=image.ny;
	ny=image.nx;
    }
    start[0]=0.0;
    start[1]=0.0;
    step[0]=1.0;
    step[1]=1.0;

    mm=(int)image.nx/2;
    if (argc==11) {
	mm=atof(ARGV[10]);
    }

/*
 * We start to trace;
 */
    sec_array = (float *)malloc((npix[1])*sizeof(float));  	
    cut = (float *)malloc((npix[1])*sizeof(float));  	
    peak_y_pixel = (int *)malloc((npix[1])*sizeof(int));  	
    peak_y_max = (float *)malloc((npix[1])*sizeof(float));  	

    fmin=1e12;
    fmax=-1e12;
    for (j=0;j<ny;j++) {
	sec_array[j]=0;
	if (width>0) {
	    for (i=(mm-width);i<(mm+width);i++) {
		if (spec_axis==0) {
		    sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		} else {
		    sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		}
	    }
	} else {
	    for (i=mm;i<(mm+1);i++) {
		if (spec_axis==0) {
		    sec_array[j]=sec_array[j]+RD_frame(&image,i,j);
		} else {
		    sec_array[j]=sec_array[j]+RD_frame(&image,j,i);
		}
	    }
	}
	if (fmin>sec_array[j]) {
	    fmin=sec_array[j];
	}
	if (fmax<sec_array[j]) {
	    fmax=sec_array[j];
	}	    
	cut[j]=1.0*j;
    }


    printf("Looking for peaks with in nsearch pixels...\n");
    npeaks=0;
    for (j=nsearch;j<(ny-nsearch);j++) {
	peak=1;
	for (i=0;i<nsearch;i++) {
	    if (sec_array[j-i]<sec_array[j-i-1]) {
		peak=0;
	    }
	    if (sec_array[j+i]<sec_array[j+i+1]) {
		peak=0;
	    }
	}
	if (peak==1) {
	    if (sec_array[j]<(imin*fmax)) {
		peak=0;
	    }
	}

	if (peak==1) {
	    if (npeaks>0) {
		delta=j-peak_y_pixel[npeaks-1];
		if (delta<dmin) {
		    peak=0;
		}
	    }
	}

	if (peak==1) {
	    peak_y_pixel[npeaks]=j;
	    a=j-1;
	    b=j;
	    c=j+1;
	    fa=-sec_array[a];
	    fb=-sec_array[b];
	    fc=-sec_array[c];
	    den=(fc-2*fb+fa);
	    if (den!=0) {
	    peak_y_max[npeaks]=c-(b-a)*((fc-fb)/den+0.5);
	    } else {
		peak_y_max[npeaks]=0;
	    }
	    npeaks++;
	}
	
    }
    printf("DONE\n");
    printf("NPEAKS=%d\n",npeaks);

    if (plot==1) {
	cpgopen("/xs");
	cpgask(0);
	cpgsfs(1.2);
	cpgscf(2);             
	cpgslw(2);
	cpgsch(1.2);
	cpgsubp(1,nplot);
	x = (float *)malloc((npeaks)*sizeof(float));  	
	y = (float *)malloc((npeaks)*sizeof(float));  	
	for (i=0;i<nplot;i++) {
	    cpgsch(1.6);
	    cpgenv((ny/nplot)*i,(ny/nplot)*(i+1),fmin,fmax,0,0);
	    cpgsch(1.3);
	    cpglab("Y-axis","Counts","");
	    cpgline(ny,cut,sec_array);    
	    cpgsch(4.0);           
	    cpgsci(2);
	    for (k=0;k<npeaks;k++) {
		x[k]=peak_y_pixel[k];
		y[k]=0.7*(fmax-fmin)+fmin;
	    }		
	    cpgpt(npeaks,x,y,5);
	    cpgsci(4);
	    for (k=0;k<npeaks;k++) {
		x[k]=peak_y_max[k];
		y[k]=0.6*(fmax-fmin)+fmin;		
	    }
	    cpgpt(npeaks,x,y,2);
	    cpgsci(3);
	    x[0]=0;
	    //	    x[1]=npix[npeaks-1];
	    x[1]=npix[1];
	    y[0]=imin*fmax;
	    y[1]=imin*fmax;
	    cpgline(2,x,y);
	    cpgsci(1);
	    cpgsch(1.2);           
	}
	free(x);
	free(y);
	cpgsci(1);
	cpgclos();
	cpgend;
    }


    printf("Writting the peaks file: %s\n",peaks_file);
    FH=fopen(peaks_file,"w");
    fprintf(FH,"#PEAK FIND\n");
    for (j=0;j<npeaks;j++) {
	fprintf(FH,"%d %d %f\n",j,peak_y_pixel[j],peak_y_max[j]);
    }
    fclose(FH);
   

    return(0);	
}


/*

}


infile=ARGV[0];
spec_axis=ARGV[1];



#if (nsearch==2*int(nsearch/2)) {
#    nsearch++;
#}

print "Reading file infile ";
nax=read_naxes(infile);   
naxis=nax;
tmp_array=read_img(infile);
print "Done\n";

#a_in=rfits(infile);
#h=a_in->gethdr;
#naxis=a_in->dims;
#tmp_array=list(a_in);

if (spec_axis==0) {
#    in_array=list(a_in);
    in_array=tmp_array;
    nx=naxis[0];
    ny=naxis[1];
} else {
    for (j=0;j<naxis[1];j++) {
	for (i=0;i<naxis[0];i++) {
	    in_array[i][j]=tmp_array[j][i];
#	    in_array[i][j]=a_in->at(i,j);#tmp_array[j][i];
	}
    }
    nx=naxis[1];
    ny=naxis[0];
}

#
# We select the central spectral regions
#
mean_point=int(nx/2);
fmin=1e12;
fmax=-1e12;

#
# We look for the peaks
#

#
# We save the result
#
open(FH,">outfile");
print FH "# PEAKS in file in_file\n";
for (j=0;j<npeaks;j++) {
    print FH "j peak_y_pixel[j] peak_y_fmax[j]\n";
}
close(FH);


#
# We plot the section
#
if (plot==1) {
    pgbegin(0,"/xs",1,1);
    pgsfs(1.2);
    pgscf(2);             # Set character font
    pgslw(2);             # Set line width
    pgsch(1.2);           # Set character height
    pgsubp(1,nplot);
    for (i=0;i<nplot;i++) {
	pgenv((ny/nplot)*i,(ny/nplot)*(i+1),fmin,fmax,0,0);
	pglabel("Y-axis","Counts","");
	pgline(ny,\cut,\sec_array);    
	pgsch(4.0);           # Set character height
	for (k=0;k<npeaks;k++) {
	    pgsci(2);
	    x=peak_y_pixel[k];
	    y=0.8*fmax;
	    pgpoint(1,[x],[y],5);
	    pgsci(4);
	    x=peak_y_fmax[k];
	    y=0.6*fmax;
	    pgpoint(1,[x],[y],2);
	}

    }
    pgsci(1);
    pgclose;
    pgend;
    print "Press Enter"; <stdin>;
}
exit;



*/
